# Dates.py - Date, DayBasis, Holiday and WorkingWeek classes
#
#		- Sudhir Shenoy, Nov., 95

#### Copyright (C) 1995-96, Sudhir Shenoy.
####
#### This library is free software; you can redistribute it and/or
#### modify it under the terms of the GNU Library General Public
#### License as published by the Free Software Foundation; either
#### version 2 of the License, or (at your option) any later version.
#### 
#### This library is distributed in the hope that it will be useful,
#### but WITHOUT ANY WARRANTY; without even the implied warranty of
#### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#### Library General Public License for more details.
#### 
#### You should have received a copy of the GNU Library General Public
#### License along with this library; if not, write to the
#### Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
#### MA 02139, USA.

import types, string, regex

DEFAULT_WORK_WEEK = [1, 1, 1, 1, 1, 0, 0]
SHORT_DAY_NAMES = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]
DAY_NAMES = ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday",
	     "Saturday", "Sunday"]
MONTH_DAYS = [0, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365]
SHORT_MONTH_NAMES = ["<1", "jan", "feb", "mar", "apr", "may", "jun", 
		     "jul", "aug", "sep", "oct", "nov", "dec", ">12"]
MONTH_NAMES = ["<1", "January", "February", "March", "April", "May", "June", 
	       "July", "August", "September", "October", "November",
	       "December", ">12"]
DMY = "dd/mm/yy"			# 29/12/1959
MDY = "mm/dd/yy"			# 12/29/1959
YMD = "yy/mm/dd"			# 1959/12/29
WMDY = "wd mm dd yy"			# Tuesday December 29 1959
SHORT_WMDY = "w mm dd yy"		# Tue December 29 1959
ALPHA_DMY = "dd mmm yy"			# 29-DEC-59

ALLOWED_FORMATS = [DMY, MDY, YMD, WMDY, SHORT_WMDY, ALPHA_DMY]
DEFAULT_DATE_FORMAT = YMD
LONG_DATE_FORMAT = WMDY
MIN_JULIAN = 2361331L
MIN_YEAR = 1753

# Global variables - work week and holidays for date arithmetic
g_week = None
g_hols = None

# Object identities
def IsDate(obj):
	return hasattr(obj, 'date_time')
def IsWorkingWeek(obj):
	return hasattr(obj, 'week') and type(obj.week) == types.ListType


class Date:				# Forward declaration
	pass


class WorkingWeek:
    """Class to hold the days of week which are considered workdays"""
    
    def __init__(self, work_week = DEFAULT_WORK_WEEK):
	err = 0
	if type(work_week) not in [types.StringType, types.TupleType,
				   types.ListType]:
	    work_week = DEFAULT_WORK_WEEK
	    err = 1
	self.week = []
	for i in range(7):
	    if i > len(work_week) or (not work_week[i]):
		self.week.append(0)
	    elif type(work_week) == types.StringType and work_week[i] == '0':
		self.week.append(0)
	    else:
		self.week.append(1)
	if err:
	    raise ValueError, "WorkingWeek: Invalid initializer"

    def __str__(self):
	rep = ""
	for i in range(7):
	    if self.week[i]:
		rep = rep + '1'
	    else:
		rep = rep + '0'
	return rep

    def __repr__(self):
	return "WorkingWeek(%s)" % repr(self.week)

    def __cmp__(self, other):
	for i in range(7):
	    if (not self.week[i]) and other.week[i]:
		return -1
	    elif self.week[i] and (not other.week[i]):
		return 1
	return 0

    def __add__(self, other):		# Union = OR
	w = []
	for i in range(7):
	    if self.week[i] or other.week[i]:
		w.append(1)
	    else:
		w.append(0)
	return WorkingWeek(w)

    def __sub__(self, other):		# Exclusive OR
	w = []
	for i in range(7):
	    if self.week[i] and other.week[i]:
		w.append(1)
	    else:
		w.append(0)
	return WorkingWeek(w)

    def isWeekday(self, i):
	if i > 6:
	    return 0
	return self.week[i]

    def isWeekend(self, i):
	if i > 6:
	    return 0
	return not(self.week[i])


class Holidays:
    """Class which holds a list of dates = holidays"""
	
    def __init__(self, holidays=None):
	self.hols = []
	if holidays == None:
	    return
	err = 0
	if type(holidays) == types.ListType:
	    for dt in holidays:
		if IsDate(dt):
		    dt = Date(julian=int(dt.date_time))
		    self.hols.append(dt)
		elif type(dt) == types.StringType:
		    dt = Date(str=dt)
		    dt.date_time = int(dt.date_time)
		    self.hols.append(dt)
	elif IsDate(holidays):
	    holidays = Date(julian=int(holidays.date_time))
	    self.hols.append(holidays)
	elif type(holidays) == types.StringType:
	    dt = Date(str = holidays)
	    dt.date_time = int(dt.date_time)
	    self.hols.append(dt)
	else:
	    raise ValueError, "Holiday initializer should be Date or list"

    def __str__(self):
	return str(self.hols)

    def __repr__(self):
	return "Holidays(%s)" % repr(self.hols)

    def __add__(self, other):
	if IsHolidays(other):
	    new_hols = self.hols[0:]
	    for x in other.hols:
		if x not in new_hols:
		    new_hols.append(x)
	    return Holidays(new_hols)

	if IsDate(other):
	    dt = Date(julian=int(other.date_time))
	elif type(other) == types.StringType and other:
	    dt = Date(str=other)
	    dt.date_time = int(dt.date_time)
	elif type(other) == types.StringType and (not other):
	    return Holidays(self.hols)
	else:
	    raise ValueError, "Only Date type can be added to Holiday list"

	new_hols = self.hols[0:]
	if dt not in new_hols:
	    new_hols.append(dt)
	return Holidays(new_hols)

    __radd__ = __add__

    def __sub__(self, other):
	if IsHolidays(other):
	    new_hols = self.hols[0:]
	    for x in other.hols:
		if x in new_hols:
		    new_hols.remove(x)
	    return Holidays(new_hols)
	if IsDate(other):
	    dt = Date(julian=other.date_time)
	elif type(other) == types.StringType and other:
	    dt = Date(str=other)
	    dt.date_time = int(dt.date_time)
	elif type(other) == types.StringType and other:
	    return Holidays(self.hols)
	else:
	    raise ValueError, "Only Date/Holidays may be subtracted from Holidays"
	new_hols = self.hols[0:]
	if dt in new_hols:
	    new_hols.remove(dt)
	return Holidays(new_hols)

    def contains(self, date):
	if IsDate(date):
	    dt = Date(julian=int(date.date_time))
	elif type(date) == types.StringType:
	    dt = Date(str=date)
	    dt.date_time = int(dt.date_time)
	else:
	    raise ValueError, "Non date type passed to Holiday.contains()"

	return dt in self.hols


class Date:
    """Date and Time class
    Internally, holds the date as a julian number + the time as a fraction of 
    day. Common date operations are defined and addition and subtraction
    operators are overloaded."""

    def __init__(self, str = None, fmt = DEFAULT_DATE_FORMAT,
		 year = None, month = None, day = None, julian=None,
		 time_str = None, hour = None, min = None, sec = None):
	if str:
	    self.date_time = Str2Date(str, fmt)
	elif julian:
	    if julian < MIN_JULIAN:	# Jan 1, 1753
		raise ValueError, 'Date before 1 Jan, 1753'
	    self.date_time = julian
	elif year and month and day:
	    self.date_time = FromYMD(year, month, day)
	else:
	    self.date_time = int(TodaysDate())

	if time_str:
	    try:
		time = Str2Time(time_str)
		# time also comes from TodaysDate()
		self.date_time = int(self.date_time) + time
	    except ValueError, err:
		time = 0
	elif hour != None:
	    try:
		time = FromHMS(hour, min, sec)
		# time also comes from TodaysDate()
		self.date_time = int(self.date_time) + time
	    except ValueError, err:
		time = 0

    def __str__(self):
	return "%s" % Date2Str(self, LONG_DATE_FORMAT)
		
    def __repr__(self, format = None):
	if format not in ALLOWED_FORMATS:
	    format = LONG_DATE_FORMAT
	return "Date(%s, %s)" % (`Date2Str(self, format)`, `format`)

    def __cmp__(self, other):
	return cmp(self.date_time, other.date_time)

    def __int__(self):
	return int(self.date_time)

    def __long__(self):
	return long(self.date_time)

    def __float__(self):
	return float(self.date_time)

    def __add__(self, other):
	if type(other) in [types.IntType, types.FloatType, types.LongType]:
	    return Date(julian = self.date_time + other)
	elif type(other) == types.StringType:
	    try:
		i, ymd = YMDString(other)
		if ymd == 'd':
		    return Date(julian = self.date_time + i)
		elif ymd == 'y':
		    i = i * 12
		return self.addMonths(i)
	    except ValueError:
		pass
	raise TypeError, "Only numbers/period strings can be added to a Date"

    __radd__ = __add__

    def __sub__(self, other):
	if IsDate(other):
	    return self.date_time - other.date_time
	elif type(other) in [types.IntType, types.FloatType, types.LongType]:
	    i = self.date_time - other
	    if i < MIN_JULIAN:
		raise OverflowError, "Date before 1 Jan, 1753"
	    return Date(julian=i)
	elif type(other) == types.StringType:
	    try:
		i, ymd = YMDString(other)
		if ymd == 'd':
		    i = self.date_time - i
		    if i < MIN_JULIAN:
			raise OverflowError, "Date before 1 Jan 1753"
		    return Date(julian=i)
		elif ymd == 'y':
		    i = i * 12
		return self.addMonths(-i)
	    except ValueError:
		pass
	raise TypeError, "Only Date/number may be subtracted from a Date"

    def __rsub__(self, other):
	if IsDate(other):
	    return other.date_time - self.date_time
	raise TypeError, "A Date can be subtracted only from another Date"

    def toYMD(self):
	"""Convert a Julian day number to its corresponding Gregorian calendar
	date. Inverse fn. of FromYMD()"""

	j = long(int(self.date_time) - 1721119L)

	yy = ((j << 2) - 1) / 146097L
	j = (j << 2) - 1 - 146097L * yy
	dd = j >> 2
	j = ((dd << 2) + 3) / 1461
	dd = (dd << 2) + 3 - 1461 * j
	dd = (dd + 4) >> 2
	mm = (5 * dd - 3) / 153
	dd = 5 * dd - 3 - 153 * mm
	dd = (dd + 5) / 5
	yy = 100 * yy + j
	if (mm < 10):
	    mm = mm + 3
	else:
	    mm = mm - 9
	    yy = yy + 1

	return (int(yy), int(mm), int(dd))

    def toHMS(self):
	secs = (self.date_time - int(self.date_time)) * 24.0 * 3600.0
	if not secs:
	    return (0, 0, 0)
	hours = int(secs / 3600.0)
	secs = secs - hours * 3600.0
	mins = int(secs / 60.0)
	secs = secs - mins * 60.0
	return (hours, mins, int(secs))

    def isLeapYear(self):
	y, m, d = self.toYMD()
	return IsLeapYear(y)

    def dayOfWeek(self):
	# Although this seems a little strange, it works.
	# (julnum + 1) % 7 gives the value 0 for Sunday..6 for Saturday.
	# Since we want the list to start at Monday, add 6 (mod 7) to this.
	# Now we have Monday at 0..Sunday at 6

	i = ((int(self.date_time) + 1) % 7 + 6) % 7
	return (i, DAY_NAMES[i])

    def nthDayOfWeek(self, n, day_of_week):
	if n > 5:
	    n = 5
	elif n < 1:
	    n = 0
	y, m, d = self.toYMD()
	dt = Date(year=y, month=m, day=1)
	day, day_name = dt.dayOfWeek()
	i = day_of_week - day
	if i < 0:
	    i = i + 7
	d = i + 1
	d = d + (n - 1) * 7
	while notIsValidDate(y, m, d):	# Overshot end of month
	    d = d - 7
	return Date(year=y, month=m, day=d)

    def isWeekend(self, week=None):
	dow, dow_name = self.dayOfWeek()
	if not (g_week or week):
	    return dow >= 5
	elif not week:
	    week = g_week
	return week.isWeekend(dow)

    def isWeekday(self, week=None):
	dow, dow_name = self.dayOfWeek()
	if not (g_week or week):
	    return dow < 5
	elif not week:
	    week = g_week
	return week.isWeekday(dow)

    def isWorkingDay(self, week=None, hols=None):
	if not (g_hols or hols):
	    return self.isWeekday(week)
	elif not hols:
	    hols = g_hols
	return self.isWeekday(week) and (not hols.contains(self))

    def prevWorkingDay(self, week=None, hols=None):
	return self.addWorkingDays(-1, week, hols)

    def nextWorkingDay(self, week=None, hols=None):
	return self.addWorkingDays(1, week, hols)

    def addWorkingDays(self, days, week=None, hols=None):
	dt = Date(julian=self.date_time)
	if not days:
	    return dt
	if days < 0:
	    inc = -1
	    days = -days
	else:
	    inc = 1
	while days:
	    dt = dt + inc
	    if dt.isWorkingDay(week, hols):
		days = days - 1
	return dt

    def diffWorkingDays(self, other, week=None, hols=None):
	if self.date_time < other.date_time:
	    dt1 = Date(julian=self.date_time)
	    dt2 = Date(julian=other.date_time)
	else:
	    dt1 = Date(julian=other.date_time)
	    dt2 = Date(julian=self.date_time)
	days = 0
	while dt1 < dt2:
	    dt1 = dt1 + 1
	    if dt1.isWorkingDay(week, hols):
		days = days + 1
	return days

    def lastWorkdayInMonth(self, week=None, hols=None):
	y, m, d = self.toYMD()
	if m in [1, 3, 5, 7, 8, 10, 12]:
	    d = 31
	elif m == 2:
	    if IsLeapYear(y):
		d = 29
	    else:
		d = 28
	elif m in [4, 6, 9, 11]:
	    d = 30
	else:
	    m = 12
	    d = 31
	dt = Date(year=y, month=m, day=d)
	if not dt.isWorkingDay(week, hols):
	    return dt.prevWorkingDay(week, hols)
	return dt

    def addMonths(self, mths, week=None, hols=None):
	y, m, d = self.toYMD()
	hh, mm, ss = self.toHMS()

	# Add months & years
	temp = m + mths
	yrs = int((temp - 1) / 12)
	y = y + yrs
	m = temp - yrs * 12
	saved_month = m

       	# If we have an impossible date (beyond end of month), move back
	while not IsValidDate(y, m, d):
	    if y < MIN_YEAR:
		raise OverflowError, "Date before 1 Jan 1753"
	    d = d - 1

	dt = Date(year=y, month=m, day=d, hour=hh, min=mm, sec=ss)
	if not dt.isWorkingDay(week, hols):
	    dt = dt.nextWorkingDay(week, hols)
	y, m, d = dt.toYMD()
	if m != saved_month:
	    dt = dt.prevWorkingDay(week, hols)

	# Lastly, check that if given date was a month-end, the computed one
	# also is the last working day of its month
	if self.lastWorkdayInMonth(week, hols) == self:
	    dt = dt.lastWorkdayInMonth(week, hols)

	return dt

# end of Date class

class DayBasis:
    """Daybasis class"""
    valid_strs = [ '30/360', '360/360', 'ACT/360', 'CNT/360', 'FIX/360',
		  '365/365', 'ACT/365']
    global DB_30_360, DB_360_360, DB_ACT_360, DB_CNT_360, DB_FIX_360
    global DB_365_365, DB_ACT_365
    DB_30_360 = valid_strs.index('30/360')
    DB_360_360 = valid_strs.index('360/360')
    DB_ACT_360 = valid_strs.index('ACT/360')
    DB_CNT_360 = valid_strs.index('CNT/360')
    DB_FIX_360 = valid_strs.index('FIX/360')
    DB_365_365 = valid_strs.index('365/365')
    DB_ACT_365 = valid_strs.index('ACT/365')

    def __init__(self, db):
	if type(db) == types.StringType:
	    db = self.fromString(db)
	self.db = db

    def fromString(self, str):
	return DayBasis.valid_strs.index(str)

    def __str__(self):
	return DayBasis.valid_strs[self.db]

    def __repr__(self):
	return "DayBasis(%s)" % self.db

    def __cmp__(self, other):
	return cmp(self.db, other)

##############################################
######## Non-member general purpose routines
##############################################

D_PAT = regex.compile("\([+-]?[0-9]+\) *[dD]\([aA][yY]\)?[sS]?$")
M_PAT = regex.compile("\([+-]?[0-9]+\) *[mM]\([oO][nN][tT][hH]\)?[sS]?$")
Y_PAT = regex.compile("\([+-]?[0-9]+\) *[yY]\([eE][aA][rR]\)?[sS]?$")

def YMDString(str):
    """Given a string of the form '1m', '3m', '2y' etc., returns a tuple 
    containing the integer part and the string 'd', 'm' or 'y'"""

    str = string.strip(str)
    if D_PAT.match(str) >= 0:
	return (string.atoi(D_PAT.group(1)), 'd')
    elif M_PAT.match(str) >= 0:
	return (string.atoi(M_PAT.group(1)), 'm')
    elif Y_PAT.match(str) >= 0:
	return (string.atoi(Y_PAT.group(1)), 'y')
    raise ValueError, "Invalid period string"

def IsLeapYear(y):
    """ Checks if a given year is a leap year"""
    return (y % 4 == 0 and (y % 100) != 0) or (y % 400 == 0)

def IsValidDate(yy, mm, dd):
    """Checks if the three numbers given form a valid date"""
    if (MIN_YEAR <= yy < 9999) and (0 < mm < 13):
	if mm == 2 and IsLeapYear(yy):
	    return (0 < dd < 30)
	else:
	    return (0 < dd <= MONTH_DAYS[mm + 1] - MONTH_DAYS[mm])
    return 0

def FromYMD(year, month, day):
    """Converts a Gregorian date to the corresponding Julian day number.
    Algorithm 199 from Communications of the ACM, Volume 6, No. 8,
    (Aug. 1963), p. 444.  Gregorian calendar started on Sep. 14, 1752.
    This function is not valid before that."""

    if not IsValidDate(year, month, day):
	raise ValueError, "Invalid date"

    if (month > 2):
	month = month - 3
    else:
	month = month + 9
	year = year - 1

    cent = long(year / 100)
    yadj = long(year - 100 * cent)
    t1 = ((146097L * cent) >> 2) + ((1461 * yadj) >> 2)
    t2 = (153 * month + 2) / 5 + day + 1721119L

    return int(t1 + t2)

def FromHMS(hour, min=None, sec=None):
    if not min:
	min = 0
    if not sec:
	sec = 0
    if 0 <= hour < 24 and 0 <= min < 60 and 0 <= sec < 60:
	temp = hour + min / 60.0 + sec / 3600.0
	return (temp / 24.0)
    raise ValueError, "Invalid time"

def TodaysDate():
    import time
    t = time.time()
    t = time.localtime(t);
    dt = FromYMD(t[0], t[1], t[2])
    hms = FromHMS(t[3], t[4], t[5])
    return dt + hms

def Time2Str(hh, mm = 0, ss = 0, fmt=None):
    if hh == 0 and mm == 0 and ss == 0:
	return ''
    suffix = ''
    if fmt == 'AM/PM':
	suffix = 'AM'
	if hh > 13:
	    hh = hh - 12
	    suffix = 'PM'
    return "%02d:%02d:%02d%s" % (hh, mm, ss, suffix)

def Str2Time(str):
    if not string.strip(str):
	return 0
    pm = 0
    str = string.lower(str)
    idx = string.find(str, 'am')
    if idx >= 0:
	str = str[:idx]
    idx = string.find(str, 'pm')
    if idx >= 0:
	str = str[:idx]
	pm = 1

    try:
	hms = string.splitfields(str, ':')
	hour = 0
	min = 0
	sec = 0
	time = 0
	if 1 <= len(hms) <= 4:
	    hour = string.atoi(hms[0])
	    if len(hms) > 1 and hms[1]:
		min = string.atoi(hms[1])
	    if len(hms) > 2 and hms[2]:
		sec = string.atof(hms[2])
	    if pm and hour < 12:
		hour = hour + 12
	    if (0 <= hour < 24) and (0 <= min < 60) and (0 <= sec < 60):
		time = (hour + min / 60.0 + sec / 3600.0) / 24.0
	    else:
		raise ValueError("Invalid values")
    except:
	raise ValueError, "Invalid string representation of Time"
    return time

def Date2Str(dt, fmt=None):
    year, month, day = dt.toYMD()
    hour, min, sec = dt.toHMS()
    if (hour, min, sec) != (0, 0, 0):
	time_suffix = ''
    else:
	time_suffix = Time2Str(hour, min, sec)
    if fmt not in ALLOWED_FORMATS:
	fmt = LONG_DATE_FORMAT
    if fmt == WMDY:
	idx, name = dt.dayOfWeek()
	str = "%s %s %d %d" % (DAY_NAMES[idx], MONTH_NAMES[month], day, year)
    elif fmt == SHORT_WMDY:
	idx, name = dt.dayOfWeek()
	str = "%s %s %d %d" % (SHORT_DAY_NAMES[idx], MONTH_NAMES[month],
			       day, year)
    elif fmt == ALPHA_DMY:
	str = "%d-%s-%d" % (day, string.upper(SHORT_MONTH_NAMES[month]),
			    year - 1900)
    elif fmt == DMY:
	str = "%d/%d/%d" % (day, month, year)
    elif fmt == MDY:
	str = "%d/%d/%d" % (month, day, year)
    elif fmt == YMD:
	str = "%d/%d/%d" % (year, month, day)
    else:
	raise RunTimeError, "Unrecognized date format"

    if time_suffix:
	return str + ' ' + time_suffix
    return str

def Str2Month(str):
    str = string.strip(str)
    try:
	return string.atoi(str)
    except ValueError:
	# Try month names
	str = string.lower(str)
	for i in range(1, 13):
	    if string.find(str, SHORT_MONTH_NAMES[i]) == 0:
		return i
    raise ValueError, "Invalid month number/string"

def Str2Date(str, fmt=None):
    if fmt not in ALLOWED_FORMATS:
	fmt = DEFAULT_DATE_FORMAT
    s = string.split(str)
    if len(s) > 1:
	if string.find(s[-1], ':') >= 0:
	    dt_str = string.join(s[:-1])
	    time = Str2Time(s[-1])
	else:
	    dt_str = str
	    time = 0
    elif len(s) == 1:
	dt_str = str
	time = 0
    else:
	raise ValueError, "Str2Date expects date or date+time"

    s = string.split(dt_str)
    if len(s) < 3:
	dt_str = string.strip(dt_str)
	s = string.splitfields(dt_str, '/')
	if len(s) < 3:
	    s = string.splitfields(dt_str, '-')
	    if len(s) < 3:
		s = string.splitfields(dt_str, ':')
    if len(s) < 3 or len(s) > 4:
	raise ValueError, "Number of components in a date string must be 3 or 4"
    if fmt == WMDY or fmt == ALPHA_DMY or fmt == SHORT_WMDY or len(s) == 4:
	if len(s) == 4:
	    first = 1
	else:
	    first = 0
	try:
	    month = Str2Month(s[first])
	    day = string.atoi(s[first + 1])
	except ValueError:
	    month = Str2Month(s[first + 1])
	    day = string.atoi(s[first])
	year = string.atoi(s[first + 2])
    elif fmt == DMY:
	try:
	    day = string.atoi(s[0])
	    month = Str2Month(s[1])
	except ValueError:
	    month = Str2Month(s[0])
	    day = string.atoi(s[1])
	year = string.atoi(s[2])
    elif fmt == MDY:
	try:
	    month = Str2Month(s[0])
	    day = string.atoi(s[1])
	except ValueError:
	    month = Str2Month(s[1])
	    day = string.atoi(s[0])
	year = string.atoi(s[2])
    elif fmt == YMD:
	year = string.atoi(s[0])
	month = Str2Month(s[1])
	day = string.atoi(s[2])
    else:
	raise RunTimeError, "Unrecognized date format"

    if year < 80:
	year = year + 2000
    elif year < 100:
	year = year + 1900

    return FromYMD(year, month, day) + time

def NumDays360(d1, d2):
    if d1 == d2:
	return 0
    elif d1 > d2:
	tmp = d1
	d1 = d2
	d2 = tmp
    y1, m1, d1 = d1.toYMD()
    y2, m2, d2 = d2.toYMD()

    if y1 == y2:
	i = 30 * (m2 - m1 - 1)
	if d1 < 30:
	    i = i + 30 - d1
	if d2 < 30:
	    i = i + d2
	else:
	    i = i + 30
    else:
	i = 360 * (y2 - y1 - 1) + 30 * (11 - m1 + m2)
	if d1 < 30:
	    i = i + 30 - d1
	if d2 < 30:
	    i = i + d2
	else:
	    i = i + 30
    return i

def NumDays365(d1, d2):
    if d1 == d2:
	return 0
    elif d1 > d2:
	tmp = d1
	d1 = d2
	d2 = tmp

    feb29s = 0
    y1, m1, d1 = d1.toYMD()
    y2, m2, d2 = d2.toYMD()
    if IsLeapYear(y1) and m1 < 3:
	feb29s = feb29s + 1
    if IsLeapYear(y2):
	if m2 > 2 or (m2 == 2 and d2 == 29):
	    feb29s = feb29s + 1
	if y1 == y2:
	    feb29s = feb29s - 1
    for y in range(y1 + 1, y2 - 1):
	if IsLeapYear(y):
	    feb29s = feb29s + 1
    return d2 - d1 - feb29s

def NumDaysByDCB(d1, d2, db, freq=0):
    if db in [DB_30_360, DB_CNT_360]:
	days = NumDays360(d1, d2)
	denom = 360.0
    elif db == DB_ACT_360:
	days = abs(d1 - d2)
	denom = 360.0
    elif db == DB_360_360:
	if not freq:
	    days = NumDays360(d1, d2)
	else:
	    days = 360.0 / freq
	i = abs(d1 - d2)
	if abs(i - days) > 30:
	    days = (i / 30) * 30
	denom = 360.0
    elif db == DB_FIX_360:

	if not freq:
	    days = NumDays360(d1, d2)
	else:
	    days = 360.0 / freq
	i = abs(d1 - d2)
	if abs(i - days) > 7:
	    days = NumDays360(d1, d2)
	denom = 360.0
    elif db == DB_ACT_365:
	days = abs(d1 - d2)
	denom = 365.0
    elif db == DB_365_365:
	days = NumDays365(d1, d2)
	denom = 365.0
    else:
	raise ValueError

    return (days, denom)

def NumYearsByDCB(d1, d2, db, freq=0):
    days, denom = NumDaysByDCB(d1, d2, db, freq)
    return days / denom

# end of Dates.py
