#! /usr/local/bin/python
#
# form.py - Demo program for TkXtra entry widgets
#

import TkXtra, DateHols
tkx=TkXtra

EXPLANATION = """This demo consists of several entry widgets. These
are useful typically in form handling applications. Two types of
constrained entry widgets are shown - one that accepts only numbers
and another that accepts only dates. A variant of each which can be
'nudged' up and down is also shown. Note that modified fields have
a different colour to provide visual feedback."""

class Demo:
    def __init__(self, root):
	self.master = root
	self.msg = tkx.Label(root, text=EXPLANATION)
	self.msg.pack(side=tkx.TOP, expand=0, fill=tkx.X)
	btns = tkx.ButtonBox(root, orient=tkx.HORIZONTAL)
	btns.add(text='Print contents', command=self.print_vals)
	btns.add(text='Dismiss', command=self.exit)
	btns.pack(side=tkx.TOP, expand=0, fill=tkx.X)

	self.le = tkx.LabelledEntry(root, label='Labelled Entry:',
				    command=self.le_changed)
	self.le.pack(side=tkx.TOP, expand=1, fill=tkx.X)

	# position title top right
	nf = tkx.LabelledFrame(root, text='Numeric Entry Fields',
			       position=tkx.N)
	nf.pack(side=tkx.TOP, expand=1, fill=tkx.BOTH)
	self.ne1 = tkx.NumericEntry(nf.frame, label='$ Amount', decimals=2,
				    command=self.calc_numbers)
	self.ne1.set('100.00')
	self.ne1.pack(side=tkx.TOP)
	self.ne2 = tkx.NumericNudgeEntry(nf.frame, label='Yen/$', decimals=2,
					 nudge=0.01, command=self.calc_numbers)
	self.ne2.set('111.52')
	self.ne2.pack(side=tkx.TOP)
	self.ne3 = tkx.NumericEntry(nf.frame, label='Yen Amount', decimals=0,
				    command=self.calc_numbers)
	self.ne3.set('11152')
	self.ne3.pack(side=tkx.TOP)

	# position title bottom left
	nf2 = tkx.LabelledFrame(root, text='Date Entry Fields',
				position=tkx.SW)
	nf2.pack(side=tkx.TOP, expand=1, fill=tkx.BOTH)
	today = DateHols.Date()
	self.de1 = tkx.DateEntry(nf2.frame, label='Date:')
	self.de1.set(str(today))
	self.de1.pack(side=tkx.TOP)
	self.de2 = tkx.DateNudgeEntry(nf2.frame, label='Date:')
	self.de2.set(str(today))
	self.de2.pack(side=tkx.TOP)

    def exit(self, event=None):
	self.master.destroy()

    def print_vals(self, event=None):
	print "USD =", self.ne1.get()
	print "Rate =", self.ne2.get()
	print "Yen =", self.ne3.get()

	print "Date 1 =", self.de1.get()
	print "Date 2 =", self.de2.get()

    def le_changed(self, widget, param, value):
	print "Labelled entry has changed value to '" + value + "'"
	return 1			# no validation - all input is OK

    def calc_numbers(self, widget, param, value):
	if widget == self.ne1:
	    yen = value * self.ne2.get()
	    self.ne3.set(yen)
	elif widget == self.ne2:
	    yen = self.ne1.get() * self.ne2.get()
	    self.ne3.set(yen)
	else:
	    usd = self.ne1.get()
	    if usd == 0:
		rate = self.ne2.get()
		if rate == 0:
		    return 1
		self.ne1.set(self.ne3.get() / rate)
	    else:
		self.ne2.set(self.ne3.get() / usd)
	return 1

def RunSample(root):
    root.minsize(10, 10)
    demo = Demo(root)
    root.mainloop()

if __name__ == '__main__':
    RunSample(tkx.Tk())
